@@ -3,11 +3,11 @@
 
 Primary class: SerialInterface
 Install with pip: "[pip3 install meshtastic](https://pypi.org/project/meshtastic/)"
-Source code on [github](https://github.com/meshtastic/Meshtastic-python)
+Source code on [github](https://github.com/meshtastic/python)
 
 properties of SerialInterface:
 
-- radioConfig - Current radio configuration and device settings, if you write to this the new settings will be applied to
+- localConfig - Current radio configuration and device settings, if you write to this the new settings will be applied to
 the device.
 - nodes - The database of received nodes.  Includes always up-to-date location and username information for each
 node in the mesh.  This is a read-only datastructure.
@@ -79,8 +79,8 @@ from google.protobuf.json_format import MessageToJson
 from meshtastic.util import fixme, catchAndIgnore, stripnl, DeferredExecution, Timeout
 from meshtastic.node import Node
 from meshtastic import (mesh_pb2, portnums_pb2, apponly_pb2, admin_pb2,
-                        environmental_measurement_pb2, remote_hardware_pb2,
-                        channel_pb2, radioconfig_pb2, util)
+                        telemetry_pb2, remote_hardware_pb2,
+                        channel_pb2, config_pb2, util)
 
 
 # Note: To follow PEP224, comments should be after the module variable.
@@ -94,7 +94,7 @@ BROADCAST_NUM = 0xffffffff
 BROADCAST_ADDR = "^all"
 """A special ID that means broadcast"""
 
-OUR_APP_VERSION = 20200
+OUR_APP_VERSION = 20300
 """The numeric buildnumber (shared with android apps) specifying the
    level of device code we are guaranteed to understand
 
@@ -181,7 +181,8 @@ protocols = {
     portnums_pb2.PortNum.NODEINFO_APP: KnownProtocol("user", mesh_pb2.User, _onNodeInfoReceive),
     portnums_pb2.PortNum.ADMIN_APP: KnownProtocol("admin", admin_pb2.AdminMessage),
     portnums_pb2.PortNum.ROUTING_APP: KnownProtocol("routing", mesh_pb2.Routing),
-    portnums_pb2.PortNum.ENVIRONMENTAL_MEASUREMENT_APP: KnownProtocol("environmental", environmental_measurement_pb2.EnvironmentalMeasurement),
-    portnums_pb2.PortNum.REMOTE_HARDWARE_APP: KnownProtocol(
-        "remotehw", remote_hardware_pb2.HardwareMessage)
+    portnums_pb2.PortNum.TELEMETRY_APP: KnownProtocol("telemetry", telemetry_pb2.Telemetry),
+    portnums_pb2.PortNum.REMOTE_HARDWARE_APP: KnownProtocol("remotehw", remote_hardware_pb2.HardwareMessage),
+    portnums_pb2.PortNum.SIMULATOR_APP: KnownProtocol("simulator", mesh_pb2.Compressed),
+    portnums_pb2.PortNum.TRACEROUTE_APP: KnownProtocol("traceroute", mesh_pb2.RouteDiscovery)
 }
