@@ -18,7 +18,7 @@ from google.protobuf.json_format import MessageToJson
 
 import meshtastic.node
 from meshtastic import portnums_pb2, mesh_pb2
-from meshtastic.util import stripnl, Timeout, our_exit, remove_keys_from_dict, convert_mac_addr
+from meshtastic.util import stripnl, Timeout, Acknowledgment, our_exit, remove_keys_from_dict, convert_mac_addr
 from meshtastic.__init__ import LOCAL_ADDR, BROADCAST_NUM, BROADCAST_ADDR, ResponseHandler, publishingThread, OUR_APP_VERSION, protocols
 
 class MeshInterface:
@@ -47,12 +47,12 @@ class MeshInterface:
         self.responseHandlers = {}  # A map from request ID to the handler
         self.failure = None  # If we've encountered a fatal exception it will be kept here
         self._timeout = Timeout()
+        self._acknowledgment = Acknowledgment()
         self.heartbeatTimer = None
         random.seed()  # FIXME, we should not clobber the random seedval here, instead tell user they must call it
         self.currentPacketId = random.randint(0, 0xffffffff)
         self.nodesByNum = None
         self.configId = None
-        self.defaultHopLimit = 3
         self.gotResponse = False # used in gpio read
         self.mask = None # used in gpio read and gpio watch
 
@@ -137,7 +137,20 @@ class MeshInterface:
                         "Latitude":  formatFloat(pos.get("latitude"),     4, "°"),
                         "Longitude": formatFloat(pos.get("longitude"),    4, "°"),
                         "Altitude":  formatFloat(pos.get("altitude"),     0, " m"),
-                        "Battery":   formatFloat(pos.get("batteryLevel"), 2, "%"),
+                    })
+                
+                metrics = node.get('deviceMetrics')
+                if metrics: 
+                    batteryLevel = metrics.get('batteryLevel')
+                    if batteryLevel is not None:
+                        if batteryLevel == 0:
+                            batteryString = "Powered"
+                        else:
+                            batteryString = str(batteryLevel)+"%"
+                        row.update({"Battery":   batteryString})
+                    row.update({
+                        "Channel util.": formatFloat(metrics.get('channelUtilization'), 2, "%"), 
+                        "Tx air util.": formatFloat(metrics.get('airUtilTx'), 2, "%"), 
                     })
 
                 row.update({
@@ -157,23 +170,24 @@ class MeshInterface:
         return table
 
 
-    def getNode(self, nodeId):
+    def getNode(self, nodeId, requestChannels=True):
         """Return a node object which contains device settings and channel info"""
         if nodeId in (LOCAL_ADDR, BROADCAST_ADDR):
             return self.localNode
         else:
             n = meshtastic.node.Node(self, nodeId)
-            logging.debug("About to requestConfig")
-            n.requestConfig()
-            if not n.waitForConfig():
-                our_exit("Error: Timed out waiting for node config")
+            # Only request device settings and channel info when necessary
+            if requestChannels:
+                logging.debug("About to requestChannels")
+                n.requestChannels()
+                if not n.waitForConfig():
+                    our_exit("Error: Timed out waiting for channels")
             return n
 
     def sendText(self, text: AnyStr,
                  destinationId=BROADCAST_ADDR,
                  wantAck=False,
                  wantResponse=False,
-                 hopLimit=None,
                  onResponse=None,
                  channelIndex=0):
         """Send a utf8 string to some other node, if the node has a display it
@@ -195,21 +209,17 @@ class MeshInterface:
         Returns the sent packet. The id field will be populated in this packet
         and can be used to track future message acks/naks.
         """
-        if hopLimit is None:
-            hopLimit = self.defaultHopLimit
 
         return self.sendData(text.encode("utf-8"), destinationId,
                              portNum=portnums_pb2.PortNum.TEXT_MESSAGE_APP,
                              wantAck=wantAck,
                              wantResponse=wantResponse,
-                             hopLimit=hopLimit,
                              onResponse=onResponse,
                              channelIndex=channelIndex)
 
     def sendData(self, data, destinationId=BROADCAST_ADDR,
                  portNum=portnums_pb2.PortNum.PRIVATE_APP, wantAck=False,
                  wantResponse=False,
-                 hopLimit=None,
                  onResponse=None,
                  channelIndex=0):
         """Send a data packet to some other node
@@ -234,8 +244,6 @@ class MeshInterface:
         Returns the sent packet. The id field will be populated in this packet
         and can be used to track future message acks/naks.
         """
-        if hopLimit is None:
-            hopLimit = self.defaultHopLimit
 
         if getattr(data, "SerializeToString", None):
             logging.debug(f"Serializing protobuf as data: {stripnl(data)}")
@@ -254,11 +262,11 @@ class MeshInterface:
         meshPacket.decoded.payload = data
         meshPacket.decoded.portnum = portNum
         meshPacket.decoded.want_response = wantResponse
+        meshPacket.id = self._generatePacketId()
 
-        p = self._sendPacket(meshPacket, destinationId,
-                             wantAck=wantAck, hopLimit=hopLimit)
         if onResponse is not None:
-            self._addResponseHandler(p.id, onResponse)
+            self._addResponseHandler(meshPacket.id, onResponse)
+        p = self._sendPacket(meshPacket, destinationId, wantAck=wantAck)
         return p
 
     def sendPosition(self, latitude=0.0, longitude=0.0, altitude=0, timeSec=0,
@@ -297,20 +305,43 @@ class MeshInterface:
                              wantAck=wantAck,
                              wantResponse=wantResponse)
 
+    def sendTraceRoute(self, dest, hopLimit):
+        """Send the trace route"""
+        r = mesh_pb2.RouteDiscovery()
+        self.sendData(r, destinationId=dest, portNum=portnums_pb2.PortNum.TRACEROUTE_APP,
+                      wantResponse=True, onResponse=self.onResponseTraceRoute)
+        # extend timeout based on number of nodes, limit by configured hopLimit
+        waitFactor = min(len(self.nodes)-1, hopLimit)
+        self.waitForTraceRoute(waitFactor)
+
+    def onResponseTraceRoute(self, p):
+        """on response for trace route"""
+        routeDiscovery = mesh_pb2.RouteDiscovery()
+        routeDiscovery.ParseFromString(p["decoded"]["payload"])
+        asDict = google.protobuf.json_format.MessageToDict(routeDiscovery)
+
+        print("Route traced:")
+        routeStr = self._nodeNumToId(p["to"])
+        if "route" in asDict:
+            for nodeNum in asDict["route"]:
+                routeStr += " --> " + self._nodeNumToId(nodeNum)
+        routeStr += " --> " + self._nodeNumToId(p["from"])
+        print(routeStr)
+
+        self._acknowledgment.receivedTraceRoute = True
+
     def _addResponseHandler(self, requestId, callback):
         self.responseHandlers[requestId] = ResponseHandler(callback)
 
     def _sendPacket(self, meshPacket,
                     destinationId=BROADCAST_ADDR,
-                    wantAck=False, hopLimit=None):
+                    wantAck=False):
         """Send a MeshPacket to the specified node (or if unspecified, broadcast).
         You probably don't want this - use sendData instead.
 
         Returns the sent packet. The id field will be populated in this packet and
         can be used to track future message acks/naks.
         """
-        if hopLimit is None:
-            hopLimit = self.defaultHopLimit
 
         # We allow users to talk to the local node before we've completed the full connection flow...
         if(self.myInfo is not None and destinationId != self.myInfo.my_node_num):
@@ -344,6 +375,8 @@ class MeshInterface:
 
         meshPacket.to = nodeNum
         meshPacket.want_ack = wantAck
+        loraConfig = getattr(self.localNode.localConfig, 'lora')
+        hopLimit = getattr(loraConfig, 'hop_limit')
         meshPacket.hop_limit = hopLimit
 
         # if the user hasn't set an ID for this packet (likely and recommended),
@@ -365,6 +398,18 @@ class MeshInterface:
         if not success:
             raise Exception("Timed out waiting for interface config")
 
+    def waitForAckNak(self):
+        """Wait for the ack/nak"""
+        success = self._timeout.waitForAckNak(self._acknowledgment)
+        if not success:
+            raise Exception("Timed out waiting for an acknowledgment")
+
+    def waitForTraceRoute(self, waitFactor):
+        """Wait for trace route"""
+        success = self._timeout.waitForTraceRoute(waitFactor, self._acknowledgment)
+        if not success:
+            raise Exception("Timed out waiting for traceroute")
+
     def getMyNodeInfo(self):
         """Get info about my node."""
         if self.myInfo is None:
@@ -421,8 +466,8 @@ class MeshInterface:
         """We need to send a heartbeat message to the device every X seconds"""
         def callback():
             self.heartbeatTimer = None
-            prefs = self.localNode.radioConfig.preferences
-            i = prefs.phone_timeout_secs / 2
+            prefs = self.localNode.localConfig
+            i = prefs.power.ls_secs / 2
             logging.debug(f"Sending heartbeat, interval {i}")
             if i != 0:
                 self.heartbeatTimer = threading.Timer(i, callback)
@@ -477,7 +522,7 @@ class MeshInterface:
         Done with initial config messages, now send regular MeshPackets
         to ask for settings and channels
         """
-        self.localNode.requestConfig()
+        self.localNode.requestChannels()
 
     def _handleFromRadio(self, fromRadioBytes):
         """
@@ -531,14 +576,50 @@ class MeshInterface:
             # stream API fromRadio.config_complete_id
             logging.debug(f"Config complete ID {self.configId}")
             self._handleConfigComplete()
+
         elif fromRadio.HasField("packet"):
             self._handlePacketFromRadio(fromRadio.packet)
+
         elif fromRadio.rebooted:
             # Tell clients the device went away.  Careful not to call the overridden
             # subclass version that closes the serial port
             MeshInterface._disconnected(self)
 
             self._startConfig()  # redownload the node db etc...
+
+        elif fromRadio.config or fromRadio.moduleConfig:
+            if fromRadio.config.HasField("device"):
+                self.localNode.localConfig.device.CopyFrom(fromRadio.config.device)
+            elif fromRadio.config.HasField("position"):
+                self.localNode.localConfig.position.CopyFrom(fromRadio.config.position)
+            elif fromRadio.config.HasField("power"):
+                self.localNode.localConfig.power.CopyFrom(fromRadio.config.power)
+            elif fromRadio.config.HasField("network"):
+                self.localNode.localConfig.network.CopyFrom(fromRadio.config.network)
+            elif fromRadio.config.HasField("display"):
+                self.localNode.localConfig.display.CopyFrom(fromRadio.config.display)
+            elif fromRadio.config.HasField("lora"):
+                self.localNode.localConfig.lora.CopyFrom(fromRadio.config.lora)
+            elif fromRadio.config.HasField("bluetooth"):
+                self.localNode.localConfig.bluetooth.CopyFrom(fromRadio.config.bluetooth)
+
+            elif fromRadio.moduleConfig.HasField("mqtt"):
+                self.localNode.moduleConfig.mqtt.CopyFrom(fromRadio.moduleConfig.mqtt)
+            elif fromRadio.moduleConfig.HasField("serial"):
+                self.localNode.moduleConfig.serial.CopyFrom(fromRadio.moduleConfig.serial)
+            elif fromRadio.moduleConfig.HasField("external_notification"):
+                self.localNode.moduleConfig.external_notification.CopyFrom(fromRadio.moduleConfig.external_notification)
+            elif fromRadio.moduleConfig.HasField("range_test"):
+                self.localNode.moduleConfig.range_test.CopyFrom(fromRadio.moduleConfig.range_test)
+            elif fromRadio.moduleConfig.HasField("telemetry"):
+                self.localNode.moduleConfig.telemetry.CopyFrom(fromRadio.moduleConfig.telemetry)
+            elif fromRadio.moduleConfig.HasField("canned_message"):
+                self.localNode.moduleConfig.canned_message.CopyFrom(fromRadio.moduleConfig.canned_message)
+            elif fromRadio.moduleConfig.HasField("audio"):
+                self.localNode.moduleConfig.audio.CopyFrom(fromRadio.moduleConfig.audio)
+            elif fromRadio.moduleConfig.HasField("remote_hardware"):
+                self.localNode.moduleConfig.remote_hardware.CopyFrom(fromRadio.moduleConfig.remote_hardware)
+
         else:
             logging.debug("Unexpected FromRadio payload")
 
@@ -631,16 +712,18 @@ class MeshInterface:
         # asObj = DotMap(asDict)
         topic = "meshtastic.receive"  # Generic unknown packet type
 
-        decoded = asDict["decoded"]
-        # The default MessageToDict converts byte arrays into base64 strings.
-        # We don't want that - it messes up data payload.  So slam in the correct
-        # byte array.
-        decoded["payload"] = meshPacket.decoded.payload
+        decoded = None
+        if 'decoded' in asDict:
+            decoded = asDict["decoded"]
+            # The default MessageToDict converts byte arrays into base64 strings.
+            # We don't want that - it messes up data payload.  So slam in the correct
+            # byte array.
+            decoded["payload"] = meshPacket.decoded.payload
 
         # UNKNOWN_APP is the default protobuf portnum value, and therefore if not
         # set it will not be populated at all to make API usage easier, set
         # it to prevent confusion
-        if "portnum" not in decoded:
+        if decoded and "portnum" not in decoded:
             new_portnum = portnums_pb2.PortNum.Name(portnums_pb2.PortNum.UNKNOWN_APP)
             decoded["portnum"] = new_portnum
             logging.warning(f"portnum was not in decoded. Setting to:{new_portnum}")
@@ -682,7 +765,8 @@ class MeshInterface:
                 # we keep the responseHandler in dict until we get a non ack
                 handler = self.responseHandlers.pop(requestId, None)
                 if handler is not None:
-                    handler.callback(asDict)
+                    if not isAck or (isAck and handler.__name__ == "onAckNak"):
+                        handler.callback(asDict)
 
         logging.debug(f"Publishing {topic}: packet={stripnl(asDict)} ")
         publishingThread.queueWork(lambda: pub.sendMessage(
